﻿using HtmlAgilityPack;
using Microsoft.VisualStudio.TestTools.WebTesting;
using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace Benday.WebLoadTest.Utilities
{
    [DisplayName("Extract Random Value From Column In Table")]
    [Description("Grabs a column value from a random row in a table.")]
    public class ExtractRandomValueFromTable : ExtractionRule
    {
        [DisplayName("Table ID")]
        [Description("Id attribute value for the table.")]
        public string TableId { get; set; }

        [DisplayName("Column Name")]
        [Description("Name of the column that should have the value.")]
        public string ColumnName { get; set; }

        public override void Extract(object sender, ExtractionEventArgs e)
        {
            if (String.IsNullOrWhiteSpace(TableId) == true)
            {
                Fail(e, "Table Id does not have a valid value.");
            }
            else if (String.IsNullOrWhiteSpace(ColumnName) == true)
            {
                Fail(e, "Column name does not have a valid value.");
            }
            else
            {
                GrabValue(e);
            }
        }

        private void GrabValue(ExtractionEventArgs e)
        {
            var doc = new HtmlAgilityPack.HtmlDocument();

            doc.LoadHtml(e.Response.BodyString);

            var table = doc.GetElementbyId(TableId);

            if (table == null)
            {
                Fail(e, String.Format("Could not locate a table tag named '{0}'.", TableId));
            }
            else if (table.Name != "table")
            {
                Fail(e, String.Format("Found a tag named '{0}' but it wasn't a table.", TableId));
            }
            else
            {
                GrabValue(e, table);
            }
        }

        private void GrabValue(ExtractionEventArgs e, HtmlNode table)
        {
            var columns = table.SelectNodes("//th");

            if (columns == null || columns.Count == 0)
            {
                Fail(e, "Could not locate any columns in the table.");
            }
            else
            {
                int columnIndex = FindColumnIndexByName(columns, ColumnName);

                if (columnIndex == -1)
                {
                    Fail(e, String.Format("Could not find a column named '{0}'.", ColumnName));
                }
                else
                {
                    GrabValue(table, columnIndex, e);
                }
            }
        }

        private static HtmlNode GetRandomRow(HtmlNodeCollection rows)
        {
            var rowCount = rows.Count;

            var randomRowIndex = new Random().Next(rowCount - 1);

            if (randomRowIndex < 0)
            {
                randomRowIndex = 0;
            }

            var row = rows[randomRowIndex];

            return row;
        }

        private bool GrabValue(HtmlNode table, int columnIndex, ExtractionEventArgs e)
        {
            bool returnValue = false;

            var rows = table.SelectNodes("//tr");

            HtmlNode cell;

            HtmlNode row = GetRandomRow(rows);

            var cells = row.SelectNodes("./td");

            if (cells == null || columnIndex >= cells.Count)
            {
                Fail(e, "No cells or index out of range.");
            }
            else
            {
                cell = cells[columnIndex];

                if (cell == null)
                {
                    Fail(e, "Cell was null.");
                }
                else
                {
                    e.WebTest.Context.Add(ContextParameterName, cell.InnerText.Trim());
                }
            }

            return returnValue;
        }

        private int FindColumnIndexByName(HtmlNodeCollection columns, string columnName)
        {
            bool foundIt = false;
            int columnIndex = 0;

            foreach (var item in columns)
            {
                if (String.Compare(item.InnerText, columnName, true) == 0)
                {
                    foundIt = true;
                    break;
                }

                columnIndex++;
            }

            if (foundIt == true)
            {
                return columnIndex;
            }
            else
            {
                return -1;
            }
        }

        private void Fail(ExtractionEventArgs e, string message)
        {
            e.Success = false;
            e.Message = message;
            e.WebTest.Context.Add(ContextParameterName, "(FAIL)");
        }
    }
}
